@@ -34,9 +34,52 @@ class SlugifiedModel(models.Model):
         super().save(*args, **kwargs)
 
 
+class ParentManager(models.Manager):
+    def get_roots(self):
+        return Parent.objects.filter(parent=None)
+
+
 class Parent(CreatedModifiedModel, SlugifiedModel):
     name = models.CharField(max_length=255)
     active = models.BooleanField()
+    parent = models.ForeignKey(
+        "Parent",
+        on_delete=models.PROTECT,
+        null=True,
+        blank=True,
+        related_name="child_parents",
+    )
+    objects = ParentManager()
+
+    class Meta:
+        ordering = ["name", "active"]
+
+    def __str__(self):
+        return self.name
+
+    def save(self, *args, **kwargs):
+        if self.parent:
+            self.active = self.parent.active
+        super().save(*args, **kwargs)
+
+    @property
+    def debit(self):
+        return self.get_sum("debit")
+
+    @property
+    def credit(self):
+        return self.get_sum("credit")
+
+    def get_sum(self, column):
+        """Returns the sum of direct child accounts and child parents."""
+        account_sum = Entry.objects.filter(
+            account__parent=self, virtual=False
+        ).aggregate(Sum(column))[f"{column}__sum"]
+        account_sum = account_sum if account_sum is not None else Decimal(0.0)
+        parent_sum = sum([p.get_sum(column) for p in self.child_parents.all()])
+        parent_sum = parent_sum if parent_sum is not None else Decimal(0.0)
+        value = account_sum + parent_sum
+        return value
 
 
 class Account(CreatedModifiedModel, SlugifiedModel):
@@ -44,8 +87,6 @@ class Account(CreatedModifiedModel, SlugifiedModel):
     parent = models.ForeignKey(
         "Parent",
         on_delete=models.PROTECT,
-        null=True,
-        blank=True,
         related_name="child_accounts",
     )
     virtual = models.BooleanField(default=False)
